@@ -24,14 +24,21 @@ if(nchar(host)>0)
   i[,5] = as.character(i[,5])
   check(x[],i)
 
+# cast
+  cast(x,x)
+
 # Selection by name
-  check(x[,"Petal_Length"][][,1], iris[,"Petal.Length"])
-  check(x$Petal_Length[][,1], iris$Petal.Length)
+  check(x[,"Petal_Length"][], iris[,"Petal.Length"])
+  check(x$Petal_Length[], iris$Petal.Length)
 # Selection along rows
-  check(x[1:5,"Petal_Length"][][,1], iris[1:5,"Petal.Length"])
+  check(x[1:5,"Petal_Length"][], iris[1:5,"Petal.Length"])
+
+# Unique of a single-attribute array
+  u = unique(x$Species)
+  check(count(u), 3)
 
 # Aggregation by a non-integer attribute with a project thrown in
   check(aggregate(iris$Petal.Length,by=list(iris$Species),FUN=mean)[,2],
-        aggregate(project(x,c('Petal_Length','Species')), by = 'Species', FUN='avg(Petal_Length)')[][,2])
+        aggregate(project(x,c('Petal_Length','Species')), by = 'Species', FUN='avg(Petal_Length)')[][,1])
 }
 gc()
